package zero1hd.rhythmbullet.audio;

import java.security.InvalidParameterException;
import java.util.Observable;
import java.util.Random;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;

public class SongListController extends Observable implements OnCompletionListener {
	private SongList songList;
	private MusicManager mdp;
	private int currentPlaybackID;
	private boolean autoPlay;
	private boolean shuffle;
	private Random rand;
	
	private Preferences prefs;
	public SongListController(SongList songList, Preferences prefs) {
		if (prefs == null) throw new NullPointerException("preferences can't be null...");
		if (songList == null) throw new NullPointerException("song list can't be null...");
		if (!songList.isSearched()) throw new InvalidParameterException("Song list has to be searched already.");
		
		this.prefs = prefs;
		this.songList = songList;
		rand = new Random();
		changeSong();
	}
	
	public void play() {
		mdp.play();
		mdp.setVolume(prefs.getFloat("music vol", 1f));
	}
	
	public void setSongByIndex(int index) {
		this.currentPlaybackID = index;
		changeSong();
	}
	
	public void skip() {
		currentPlaybackID++;
		changeSong();
	}
	
	public void previous() {
		currentPlaybackID--;
		changeSong();
	}

	@Override
	public void onCompletion(Music music) {
		if (autoPlay) {
			if (shuffle) {
				if (songList.getAmountOfSongs() != 0) {
					currentPlaybackID = rand.nextInt(songList.getAmountOfSongs());
				} else {
					currentPlaybackID = 0;
				}
			} else {
				currentPlaybackID++;
				if (currentPlaybackID > songList.getAmountOfSongs()) {
					currentPlaybackID = 0;
				}
			}
			changeSong();
			play();
		}
	}
	
	public void shuffle() {
		if (songList.getAmountOfSongs() == 0) {
			currentPlaybackID = 0;
		} else {
			currentPlaybackID = rand.nextInt(songList.getAmountOfSongs());
		}
		changeSong();
	}
	
	public void setAutoPlay(boolean autoPlay) {
		this.autoPlay = autoPlay;
	}
	
	public void setShuffle(boolean shuffle) {
		this.shuffle = shuffle;
	}
	
	public boolean isShuffle() {
		return shuffle;
	}
	
	public boolean isAutoPlay() {
		return autoPlay;
	}
	
	private void changeSong() {
		if (mdp != null) {
			mdp.dispose();
		}
		
		if (shuffle) {
			if (songList.getAmountOfSongs() == 0) {
				currentPlaybackID = 0;
			} else {
				currentPlaybackID = rand.nextInt(songList.getAmountOfSongs());
			}
		} else {
			if (currentPlaybackID > songList.getAmountOfSongs() -1) {
				currentPlaybackID = 0;
			}
		}
		if (currentPlaybackID < 0) {
			currentPlaybackID = songList.getAmountOfSongs();
		}
		this.mdp = songList.getMusicInfoFromIndex(currentPlaybackID);
		if (mdp == null) {
			if (!Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3").exists()) {
				Gdx.files.internal("music/Alan Walker - Spectre.mp3").copyTo(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
			}
			mdp = songList.getAudioData(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
		}
		mdp.setOnCompletionListener(this);
		
		setChanged();
		notifyObservers(mdp);
	}
	
	public SongList getSongList() {
		return songList;
	}
	
	
	public MusicManager getCurrentSong() {
		return mdp;
	}
}
