package zero1hd.rhythmbullet.audio;

import java.io.IOException;
import java.security.InvalidParameterException;

import javax.sound.sampled.AudioFormat;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;
import com.badlogic.gdx.files.FileHandle;

import zero1hd.rhythmbullet.audio.wavedecoder.WavDecoder;

public class WAVManager implements MusicManager {
	private int readWindowSize = 1024;
	private AudioFormat format;
	private int readIndex;
	private int currentReadWindowIndex;
	private Music playbackMusic;
	WavDecoder decoder;
	
	private String basicSongName;
	public WAVManager(FileHandle file) {
		basicSongName = file.name();
		try {
			decoder = new WavDecoder(file);
		} catch (InvalidParameterException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		playbackMusic = Gdx.audio.newMusic(file);
		format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, (float) decoder.getSampleRate(), 16, decoder.getChannels(), decoder.getChannels()*2, (float) decoder.getSampleRate(), false);
	}
	
	@Override
	public void playbackIndexUpdate() {
		readIndex = (int) (playbackMusic.getPosition() * decoder.getSampleRate() / readWindowSize);
	}
	
	@Override
	public int getPlaybackIndexPosition() {
		return readIndex;
	}
	
	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public int readSamples(float[] samples) {
		int samplesRead = 0;
		try {
			samplesRead = decoder.readSamples(samples);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (samplesRead != 0) {
			currentReadWindowIndex++;
		}
		return samplesRead;
	}

	@Override
	public long getSampleCount() {
		return decoder.getFrameLength();
	}

	@Override
	public void dispose() {
		playbackMusic.stop();
		playbackMusic.dispose();
		decoder.cleanAndClose();
	}

	@Override
	public float getDuration() {
		return decoder.getDurationInSeconds();
	}

	@Override
	public float getSampleRate() {
		return format.getSampleRate();
	}

	@Override
	public int getCurrentReadWindowIndex() {
		return currentReadWindowIndex;
	}

	@Override
	public void pause() {
		playbackMusic.pause();
	}

	@Override
	public void play() {
		playbackMusic.play();
	}

	@Override
	public boolean isPlaying() {
		return playbackMusic.isPlaying();
	}

	@Override
	public float getPositionInSeconds() {
		return playbackMusic.getPosition();
	}

	@Override
	public void setPosition(float position) {
		playbackMusic.setPosition(position);
	}

	@Override
	public void setOnCompletionListener(OnCompletionListener listener) {
		playbackMusic.setOnCompletionListener(listener);
	}

	@Override
	public void setVolume(float percent) {
		playbackMusic.setVolume(percent);
	}

	@Override
	public boolean isFinishedLoading() {
		return true;
	}
	
	@Override
	public String getBasicSongName() {
		return basicSongName;
	}
}
