package zero1hd.rhythmbullet.graphics.ui.components;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.ui.Widget;

import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.visualizer.BasicVisualizer;

public class Visualizer extends Widget {
	private BasicVisualizer vis;
	private boolean updatePositioning = true;
	private boolean mmSet;
	private MusicManager mm;
	public Visualizer() {
		vis = new BasicVisualizer();
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		vis.render(batch, parentAlpha);
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void act(float delta) {
		if (mm != null && mm.isFinishedLoading() && !mmSet) {
			Gdx.app.debug("Visualizer", "\nsample count: " + mm.getSampleCount()
			+ "\nDuration in seconds: " + mm.getDuration() + 
			"\nSample rate: " + mm.getSampleRate());
			vis.setMM(mm);
			mmSet = true;
		}
		
		vis.modify(delta);
		setVisualizerPosProp();
		if (updatePositioning) {
			vis.updatePositionInfo();
			vis.setxPos((getWidth() - vis.getActualWidth())/2f);
		}
		vis.calculate();
		super.act(delta);
	}
	
	
	public void setMM(MusicManager mm) {
		this.mm = mm;
		mmSet = false;
	}
	
	@Override
	public void setColor(Color color) {
		vis.setColor(color);
		super.setColor(color);
	}
	
	@Override
	public void setColor(float r, float g, float b, float a) {
		vis.setColor(r, g, b, a);
		super.setColor(r, g, b, a);
	}
	
	public void setVisualizerPosProp() {
		vis.setHeight(getHeight());
		vis.setWidth(getWidth());
		vis.setxPos(getX());
		vis.setyPos(getY());
		vis.setRotation(getRotation());
	}
	
	public void setUpdatePositioning(boolean updatePositioning) {
		updateVisualPosition();
		this.updatePositioning = updatePositioning;
	}
	
	public void updateVisualPosition() {
		setVisualizerPosProp();
		vis.updatePositionInfo();
		vis.setxPos(((vis.getWidth() - vis.getActualWidth())/2f));
		Gdx.app.debug("Visualizer", "currently offseting visualizer by (px): " + vis.getxPos());
		vis.updatePositionInfo();
	}
	
	public boolean isUpdatePositioning() {
		return updatePositioning;
	}
	
	public BasicVisualizer getVis() {
		return vis;
	}
	
}
