package zero1hd.rhythmbullet.graphics.ui.pages;

import java.util.Observable;
import java.util.Observer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.SongListController;
import zero1hd.rhythmbullet.graphics.ui.components.MusicControls;
import zero1hd.rhythmbullet.graphics.ui.components.ScrollText;
import zero1hd.rhythmbullet.graphics.ui.components.TitleBarVisualizer;
import zero1hd.rhythmbullet.screens.MainMenu;

public class MainPage extends Page implements Observer {
	private Label versionLabel;
	
	private SongListController sc;
	private TitleBarVisualizer titleBar;
	private Table table;
	private TextButton playButton;
	private TextButton optionsButton;
	private TextButton quitButton;
	
	private MusicControls musicControls;
	
	private ScrollText scrollText;
	public MainPage(RhythmBullet core, Vector3 targetPosition, SongListController sc, MainMenu mm) {
		this.sc = sc;
		setTextureBackground(core.getAssetManager().get("gradients.atlas", TextureAtlas.class).findRegion("red-linear"));
		
		titleBar = new TitleBarVisualizer(core.getAssetManager());
		addActor(titleBar);
		
		versionLabel = new Label("Version: " + RhythmBullet.VERSION, core.getDefaultSkin(), "sub-font",
				core.getDefaultSkin().getColor("default"));
		versionLabel.setPosition(3, 3);
		addActor(versionLabel);
		
		table = new Table();
		table.setSize(getWidth(), titleBar.getY());
		table.align(Align.center);
		table.defaults().space(10f);
		addActor(table);
		
		playButton = new TextButton("Start!", core.getDefaultSkin());
		playButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.x = Gdx.graphics.getWidth()*1.5f;
				getStage().setKeyboardFocus(mm.getMusicSelectionPage());
			}
		});
		table.add(playButton).width(Gdx.graphics.getWidth()*0.2f);
		
		table.row();
		
		optionsButton = new TextButton("Options", core.getDefaultSkin(), "sub");
		optionsButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.x = Gdx.graphics.getWidth()*-0.5f;
			}
		});
		table.add(optionsButton).fillX();
		
		table.row();
		
		quitButton = new TextButton("Quit", core.getDefaultSkin(), "sub");
		quitButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				Gdx.app.exit();
			}
		});
		table.add(quitButton).fillX();
		
		musicControls = new MusicControls(core.getDefaultSkin(), sc);
		musicControls.setPosition((getWidth()-musicControls.getMinWidth() - 20f), getHeight()-musicControls.getMinHeight()-15f);
		addActor(musicControls);
		
		scrollText = new ScrollText("...", "...", core.getDefaultSkin(), false, true);
		scrollText.setWidth(0.5f*getWidth());
		scrollText.setPosition(15, getHeight() - scrollText.getHeight()-15f);
		addActor(scrollText);
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
	}

	@Override
	public void dispose() {
		super.dispose();
	}

	@Override
	public void update(Observable o, Object arg) {
		if (o == sc) {
			MusicManager mm = (MusicManager) arg;
			updateVisualsForDifferentSong(mm);
		}
	}
	
	public void updateVisualsForDifferentSong(MusicManager mm) {
		titleBar.getHvisual().setMM(mm);
		scrollText.setText(mm.getBasicSongName(), null);
	}
}
